@@ -2,25 +2,26 @@
 use warnings;
 use Noactions;
 
-sub NUM::action {
+sub Post::NUM::action {
   return $_[1];
 }
 
-sub PLUS::action {
+sub Post::PLUS::action {
   "$_[1] $_[3] +";
 }
 
-sub TIMES::action {
+sub Post::TIMES::action {
   "$_[1] $_[3] *";
 }
 
-my $parser = Noactions->new();
+my $debug = shift || 0;
+my $parser = Noactions->new( yyprefix => 'Post::');
 print "Write an expression: "; 
 my $x;
 {
   local $/ = undef;
-  $x = <>;
+  $x = <STDIN>;
 }
-my $t = $parser->Run($x);
+my $t = $parser->Run($x, $debug);
 
 print "$t\n";
